#include "Logistic.h"
#include <iostream>
#include <cmath>
#include <fstream>
#include <string>
#include <sstream>
#include <vector>
#include <algorithm>
#include "newick.h"
#include "Tree.h" 
#include <stdio.h>      /* printf, scanf, puts, NULL */
#include <stdlib.h>     /* srand, rand */
#include <time.h>  

int GetM(string xFileName)
{
	ifstream f;
    f.open(xFileName.c_str());
    string text = "";
    getline(f, text); 
    istringstream oss(text);
    string word;
    int _M = 0;
    while(oss >> word)
     	_M ++;
    return _M - 1;
    f.close();
}

int GetN(string xFileName)
{
	ifstream f;
    f.open(xFileName.c_str());
    string text = "";
    int _N = 0;
    while (!f.eof())
    {
        getline(f, text); 
        if (text.length() > 0 ) _N ++;;
    }
   
    f.close();
    return _N;
}



int GetC(string yFileName)
{
	ifstream f;
    f.open(yFileName.c_str());
    string text = "";
    int _C = 0;
    while (!f.eof())
    {
        getline(f, text); 
        if (text.length() > 0 && text[0] == '>') _C++;
    }
    f.close();
    return _C;
}

//Data::~Data()
//{
	
	/*delete[] X;
	delete[] y;
	delete[] Y;*/

//}

void Data::GetData(string xFileName, vector <int> idsTrain)
{
	X = new float [N * M];
	
    ifstream f;
    f.open(xFileName.c_str());
    string text = "";
    int counter = 0;
    int nline = 0;
    while (!f.eof())
    {
        getline(f, text); 
        if (!idsTrain.size() || count(idsTrain.begin(), idsTrain.end(), nline))
        {

			istringstream oss(text);
    		float word;   
    		oss >> word;
    		while(oss >> word)
    		{
    			
     			if (counter < M * N) X[counter] = word;
     			//else cout << "Wrong Input X" << endl;
     			counter++;
     		}
     		
     	}
     	
     	nline ++;

    }
    totalN = nline;
    f.close();
}

void Data::normalize()
{
	for (int i = 0; i < N; i ++)
	{
		float sum = 0;
		for (int j = 0; j < M; j ++)
			sum += getX(i,j);	
		for (int j = 0; j < M; j ++)
			setX(i,j, float(getX(i,j)) / sum);	
	}
}

void Data::getClasses(string yFileName, vector <int> idsTrain)
{
    int *_Y = new int [totalN];
    y = new int [N];
    ifstream f;
    f.open(yFileName.c_str());
    string text = "";
    int curC = 0;
    while (!f.eof())
    {
        getline(f, text); 
        if (text.length() > 0 && text[0] == '>') 
        {

        	curC ++;
        	getline(f, text);
        	istringstream oss(text);
    		int word;   
    			
    		while(oss >> word)
    		{
    			_Y[word - 1] = curC;   		
     		} 
        }
    }
   
    f.close();
    int counter = 0;
    
    for (int i = 0; i < totalN - 1; i++)
    {
    	if (!idsTrain.size() || count(idsTrain.begin(), idsTrain.end(), i))
    	{
    		if (counter < N) y[counter] = _Y[i];
     		//else cout << "Wrong Input Y" << endl;   		
    		counter ++;
    	}
    }
    Y = new int [C * N];
    for (int i = 0; i < C * N; i++) Y[i] = 0;
    
    
    for (int i = 0; i < N; i++) Y[i * C + y[i] - 1] = 1; 
    delete [] _Y;  
}

Data::Data(){ X = NULL; y = NULL;}
Data::Data(string xFileName, string yFileName, vector <int> idsTrain)
{
	
	if (idsTrain.size()) N = idsTrain.size();
	else N = GetN(xFileName);
	M = GetM(xFileName);
	C = GetC(yFileName);

	GetData(xFileName, idsTrain);
	getClasses(yFileName, idsTrain);
	normalize();	
	
}

Data::Data(string xFileName, vector <int> idsTrain)
{
	
	if (idsTrain.size()) N = idsTrain.size();
	else N = GetN(xFileName);
	M = GetM(xFileName);
	
	GetData(xFileName, idsTrain);
	normalize();	
}

void Logistic::del()
{
         
		delete[] Beta;
		delete[] Delta;
		delete[] NonZeroData;
	
		delete[] llevel;	
		delete[] ddist;
		delete[] height;
		delete[] W;
		delete[] W_leaf;

		delete[] s;
		delete[] g;
		delete[] MultS;
		delete[] ScalarMult; 
		delete[] NumBetas;
		delete[] A;
		delete[] r1_plus;
		delete[] r1_minus;
		delete[] r2;
        delete[] E;
        delete[] XY;	
        delete[] Training.X;
		delete[] Training.y;
		delete[] Training.Y;
		delete tree;
}

Logistic::Logistic(string xFileName, string yFileName, string tFileName, vector <int> idsTrain)
{
	
	Training = Data(xFileName, yFileName, idsTrain); 
	
	spidir::Tree * nulltree = NULL;
	tree = readNewickTree(tFileName.c_str(), nulltree);

	root =  tree -> root;
	M = Training.M;
	N = Training.N;
	C = Training.C;


	Beta = NULL;
	Delta = NULL;
	NonZeroData = NULL;
	
	llevel = NULL;	
	ddist = NULL;	
	height = NULL;	
	W = NULL;	
	W_leaf = NULL;	
	s = NULL;	
	g = NULL;	
	MultS = NULL;	
	ScalarMult = NULL;	 
	NumBetas = NULL;	
	A = NULL;	
	r1_plus = NULL;	
	r1_minus = NULL;	
	r2 = NULL;	
    E = NULL;	
    XY = NULL;	

}

Logistic::Logistic(string xFileName, vector <int> idsTrain)
{
	
	Training = Data(xFileName, idsTrain); 

	spidir::Tree * nulltree = NULL;

	M = Training.M;
	N = Training.N;

   
	Beta = NULL;
	Delta = NULL;
	NonZeroData = NULL;
	
	llevel = NULL;	
	ddist = NULL;	
	height = NULL;	
	W = NULL;	
	W_leaf = NULL;	
	s = NULL;	
	g = NULL;	
	MultS = NULL;	
	ScalarMult = NULL;	 
	NumBetas = NULL;	
	A = NULL;	
	r1_plus = NULL;	
	r1_minus = NULL;	
	r2 = NULL;	
    E = NULL;	
    XY = NULL;	

}

void Logistic::Train(float _lambd)
{
	Lambd = _lambd;
	Beta = new float [C * M];
	for (int i = 0; i < C * M; i++) Beta[i] = 0;   

    NonZeroData = new vector <int> [M];
	for (int i = 0; i < M; i++) 
		for (int j = 0; j < N; j++) 
			if (Training.getX(j,i) != 0)
				NonZeroData[i].push_back(j);       
			
	Delta = new float [C * M];
	for (int i = 0; i < C * M; i++) Delta[i] = 1;   

	MegaSum = 0;
	Penalty = 0;
}

void Logistic::TreePass(spidir::Node *node, int level, float dist)
{
	vector <int> kidsLocal;
	vector <int> leavesLocal;
	int NodeId = NInterNodes;
	kids.push_back(kidsLocal);
	leaves.push_back(leavesLocal);
	Level.push_back(0);
	Dist.push_back(0);

    for (int i = 0; i <  node -> nchildren; i++)
    {
    	spidir::Node * kid = node -> children[i];
    	if  (kid -> isLeaf())
    	{ 
    		
    		int kidName = atoi((kid->longname).c_str());
			leavesLocal.push_back(kidName);
            llevel[kidName] = level + 1;			
			ddist[kidName] = node-> dist;
		}
		else
		{
			NInterNodes += 1;
			kidsLocal.push_back(NInterNodes);
			TreePass(kid, level + 1, dist + node->dist);
		}
    }
					
	kids[NodeId] = kidsLocal;
	leaves[NodeId] = leavesLocal;
	Level[NodeId] = level;
	Dist[NodeId] = dist;
	for (int i = 0; i < leavesLocal.size(); i++)
		J.push_back(leavesLocal[i]);	
}

float Logistic::TreePassHeight(int node)
{
	vector <float> dist;
	for (int i = 0; i < kids[node].size(); i++)
		dist.push_back(TreePassHeight(kids[node][i]));
	for (int i = 0; i < leaves[node].size(); i++)
		dist.push_back(ddist[i]);
	
	height[node] = *max_element(dist.begin(), dist.end());
	return height[node] + Dist[node];
}

void Logistic::Weights(int node, int level)
{
	W[node] = MultS[node] * g[node];
	
    for (int i = 0; i < kids[node].size(); i++)
    {
        
		MultS[kids[node][i]] = MultS[node] * s[node];
		Weights(kids[node][i], level + 1);
		
	}
	for (int i = 0; i < leaves[node].size(); i++)
	{
		
		W_leaf[leaves[node][i]] = MultS[node] * s[node];
		
	}

}

void Logistic::LoadTheTree(float wScheme)
{
		NInterNodes = 0;
		llevel = new int [M];
		ddist = new float [M];
		

		TreePass(root, 0, 0);
		NInterNodes += 1;
		
		sort(J.begin(), J.end());
		

		height = new float [NInterNodes];

		TreePassHeight(0);
	
        
		MultS = new float [NInterNodes];
		for (int i = 0; i < NInterNodes; i++) MultS[i] = 1 ;
		W = new float [NInterNodes];
		W_leaf = new float [M];
		s = new float [NInterNodes];
		g = new float [NInterNodes];
		
		if (wScheme >= 0)
		{
			for (int i = 0; i < NInterNodes; i++)
			{
				s[i] = wScheme;
				g[i] = 1 - wScheme;
			}
		}
		else if (wScheme == -1)
		{
			float H = 0;
			for (int i = 0; i < NInterNodes; i++) if (height[i] > H) H = height[i];
			//float H = * max_element(height.begin(), height.end());
			for (int i = 0; i < NInterNodes; i++)
			{
				s[i] = height[i] / H;
				g[i] = 1 - s[i];
			}
		} 
		else if (wScheme == -2)
                {
                        float H = 0;
                        for (int i = 0; i < NInterNodes; i++) if (height[i] > H) H = height[i];
                        //float H = * max_element(height.begin(), height.end());
                        for (int i = 0; i < NInterNodes; i++)
                        {
                                g[i] = height[i] / H;
                                s[i] = 1 - s[i];
                        }
                }
                
      
		Weights(0, 0);
	
}

void Logistic::Scalar()
{
	ScalarMult = new float [C * N];
	for (int i = 0; i < C * N; i ++) ScalarMult[i] = 0;
}

void Logistic::PartialSquareSumsRecursion(int v)
{

	for (int i = 0; i < leaves[v].size(); i++)
		for (int c = 0; c < C; c++)
			setA(c,v,getA(c,v) + getBeta(c,leaves[v][i]) * getBeta(c,leaves[v][i]));
	
	for (int i = 0; i < kids[v].size(); i++)
    {
    	PartialSquareSumsRecursion(kids[v][i]);
		for (int c = 0; c < C; c++)
			setA(c,v,getA(c,v) + getA(c,kids[v][i]));		
	}	

}	

void Logistic::PartialSquareSums()
{
	for (int i = 0; i < NInterNodes * C; i++) A[i] = 0;
	PartialSquareSumsRecursion(0);
}

void Logistic::TreePassNumBetas(int node)
{
	for (int i = 0; i < kids[node].size(); i++)
    {
    	TreePassNumBetas(kids[node][i]);
		for (int c = 0; c < C; c++)
			setNumBetas(node,c,getNumBetas(node,c) + getNumBetas(kids[node][i],c));
	}	
	for (int i = 0; i < leaves[node].size(); i++)
		for (int c = 0; c < C; c++)
			if (getBeta(c,leaves[node][i]) != 0)
				setNumBetas(node,c, getNumBetas(node,c) + 1);
}	

void Logistic::TreePassSum(int node, int c, float X1, float X2, float Y1, float Y2, float Z, float ZZ)
{
	/*
		kids = self.kids[node]
		leaves = self.leaves[node]
	*/
    float sa, sacube, a1, a2, a3, b1;
    if (getNumBetas(node,c) >= 0)
    {
    	sa = sqrt(getA(c,node));
    	sacube = sa * getA(c,node);
    	if (sa != 0) sa =  1.0 / sa;
    	else sa = 0; //10
    	if (sacube != 0) sacube =  1.0 / sacube;
    	else sacube = 0; //10
    	
    }
    if (getNumBetas(node,c) > 1)
    {
    	X1 = X1 + sa * W[node];
		X2 = X2 + sacube * W[node];
    }
    if (getNumBetas(node,c) == 1)
    {
    	Y1 = Y1 + sa * W[node];
		Y2 = Y2 + sacube * W[node];
		Z = Z + W[node];
    }
    if (getNumBetas(node,c) == 0)
    {
    	ZZ = ZZ + W[node]; 
    }
    for (int i = 0; i < kids[node].size(); i++)
    {
    	TreePassSum(kids[node][i], c, X1, X2, Y1, Y2, Z, ZZ);
	}
	for (int i = 0; i < leaves[node].size(); i++)
    {
    	if ( getBeta(c,leaves[node][i]) != 0)
    	{
    			a1 = X1 * getBeta(c,leaves[node][i]);
				b1 = Z + W_leaf[leaves[node][i]];
				a2 = X2 * getBeta(c,leaves[node][i]) * getBeta(c,leaves[node][i]);
				a3 = X1	;
    	}
    	else
    	{
    		    a1 = (X1 + Y1) * getBeta(c,leaves[node][i]);
				b1 = ZZ + W_leaf[leaves[node][i]];
				a2 = (X2 + Y2) * getBeta(c,leaves[node][i]) * getBeta(c,leaves[node][i]);
				a3 = (X1 + Y1);
    	}
    	setr1_plus(c,leaves[node][i],(a1 + b1) * Lambd);
		setr1_minus(c,leaves[node][i],(a1 - b1) * Lambd);
		setr2(c,leaves[node][i],(-a2 + a3) * Lambd);

    }
}		

void Logistic::Derivatives()
{
	for (int i = 0; i < NInterNodes * C; i++) NumBetas[i] = 0;
	PartialSquareSums();
    TreePassNumBetas(0);
    for (int i = 0; i < M * C; i++)
    {
    	r1_plus[i] = 0;
    	r1_minus[i] = 0;
    	r2[i] = 0;
    }
    for (int c = 0; c <  C; c++)
    	TreePassSum(0, c, 0, 0, 0, 0, 0, 0);

}

void Logistic::dl1(int k, int j, float &a, float &c)
{
		float s1 = 0.0;
		float s2 = 0.0;
		s1 =  -getXY(j,k);
	
		for (int i = 0; i < NonZeroData[j].size(); i++)
		{
		    float t = E[NonZeroData[j][i]];
		   
		    
			if (t != 0) t = 1.0 / t; 
			float z = getScalarMult(k,NonZeroData[j][i]);
			
			t = t * exp(z);			
			t = t * Training.getX(NonZeroData[j][i],j);	

			
			s2 += - t * t + t * Training.getX(NonZeroData[j][i],j);				
			s1 += t ;
			
		}
		a = s1;
		c = s2 ;

}

float Logistic::compute(int k, int j)
{

	float a, c, b, d, e;
	
	dl1(k,j, a, c);
	b = getr1_plus(k,j);
	d = getr2(k,j); 
	e = getr1_minus(k,j);

	float res = 0;
	if (getBeta(k,j) > 0)
	{
			if (abs(c + d) !=0)
				res = -(a + b) / (c + d) ;
			else res = 0;			
			if (getBeta(k,j) + res  < 0) 
				res = -getBeta(k,j);		
	}	

	if (getBeta(k,j) < 0)
	{
			if (abs(c + d) !=0)
				res = -(a + e) / (c + d) ;
			else res = 0;			
			if (getBeta(k,j) + res  > 0) 
				res = -getBeta(k,j);		
	}	

	if (getBeta(k,j) == 0)
	{
		if (abs(c + d) != 0)
		{
				float res1 = -(a + b) / (c + d) ;
				float res2 = -(a + e) / (c + d);
				float e0 = eval(k,j,0);
				float eplus = eval(k,j,res1);
				float eminus = eval(k,j,res2);
				
				if (eplus < e0 && eplus <= eminus)
					res = res1;
				else if( eminus < e0 && eminus < eplus)
					res = res2;
				else res = 0;
		}
	}	
	
	return res;		
 	
 
}

float Logistic::eval(int K, int J, float step)
{
		float dbeta = min(max(step, -getDelta(K,J)), getDelta(K,J));
		float newBeta = getBeta(K,J) + dbeta;
		
		float MegaSumLocal = 0;
		if (dbeta != 0)
		{
			for (int i = 0; i < NonZeroData[J].size(); i++)
			{
				MegaSumLocal += dbeta * Training.getX(NonZeroData[J][i],J) * Training.getY(NonZeroData[J][i],K);
				E[NonZeroData[J][i]] = E[NonZeroData[J][i]] - exp(getScalarMult(K,NonZeroData[J][i])) +  exp(getScalarMult(K,NonZeroData[J][i]) + dbeta * Training.getX(NonZeroData[J][i],J));
			}			
		}
		float s1 = MegaSum + MegaSumLocal;
		s1 = -s1;
		for (int i = 0; i < N; i++)	
		{					
			float t = E[i];
			t = log(t); 
			//if (t != t) cout << "error";
			s1 += t;
		}
		float s2 = Penalty + W_leaf[J] * (-abs(newBeta - dbeta) + abs(newBeta));
		s1 = s1 + s2 * Lambd;

		
		if (dbeta != 0)
			for (int i = 0; i < NonZeroData[J].size(); i++)
			{
				//if ()
				E[NonZeroData[J][i]] = E[NonZeroData[J][i]] + exp(getScalarMult(K,NonZeroData[J][i])) -  exp(getScalarMult(K,NonZeroData[J][i]) + dbeta * Training.getX(NonZeroData[J][i],J));
			    //if (E[NonZeroData[J][i]] != E[NonZeroData[J][i]]) cout << "error";
			}
		return s1;
}

void Logistic::Learn(float wScheme)
{

	unsigned t0 = clock(), t1;

	LoadTheTree(wScheme);
	Scalar();
	float Eval = 0;
	NumBetas = new float [NInterNodes * C];
	A = new float [NInterNodes * C];
	r1_plus = new float [C * M];
    r1_minus = new float [C * M];
    r2 = new float [C * M];
    E = new float[N];
    for (int i = 0; i < N; i++) E[i] = C;

    XY = new float [M * C];
	for (int i = 0; i < M * C; i++) XY[i] = 0;

	for (int jj = 0; jj < J.size(); jj++)
	{  int j = J[jj];
      for (int k = 0; k < C; k++)
      {
    	for (int i = 0; i < NonZeroData[j].size(); i++)
    	{
			 addXY(j,k,Training.getX(NonZeroData[j][i],j) * Training.getY(NonZeroData[j][i],k));
		}

	  }
	}
	
	t1 = clock()- t0;
	t0 = clock();

	for (int t = 0; t < 500; t ++)
	{
			Derivatives();
		
			float EvalNew = eval(0,0,0);
			if (EvalNew != EvalNew) break;
			if (abs(EvalNew - Eval) < 0.00001)
				if (t >= 10) break;
			if (EvalNew > Eval) //break;
				if (t >= 1) break;
			

			Eval = EvalNew;
			//cout << t << " " << Eval << " ";

			for (int jj = 0; jj < J.size(); jj++)
	{  int j = J[jj];
      for (int k = 0; k < C; k++)
      {
    	for (int i = 0; i < NonZeroData[j].size(); i++)
    	{
			// addXY(j,k,Training.getX(NonZeroData[j][i],j) * Training.getY(NonZeroData[j][i],k));
		}
		//cout << j << " " << k << " " << getXY(j,k) << endl;
	  }
	}
			for (int jj = 0; jj < J.size(); jj++)
				//if (J[jj] != 10)
				for (int k = 0; k < C; k++)
			{
				int j = J[jj];
				float step = compute(k,j);	
//cout << j << " " << k << " " << step  << endl;
				float dbeta = min(max(step, -getDelta(k,j)), getDelta(k,j));
				float newBeta = getBeta(k,j) + dbeta;

				if (dbeta != 0)
				{
					Penalty = Penalty + W_leaf[j] * (-abs(getBeta(k,j)) + abs(newBeta));	
					for (int i = 0; i < NonZeroData[j].size(); i++)
					{
						float ss = getScalarMult(k,NonZeroData[j][i]);
						setScalarMult(k,NonZeroData[j][i], getScalarMult(k,NonZeroData[j][i]) + dbeta * Training.getX(NonZeroData[j][i],j) );
						MegaSum += dbeta * Training.getX(NonZeroData[j][i],j) * Training.getY(NonZeroData[j][i],k);
						E[NonZeroData[j][i]] = E[NonZeroData[j][i]] - exp(ss) +  exp(getScalarMult(k,NonZeroData[j][i]));
						//if (E[NonZeroData[j][i]] != E[NonZeroData[j][i]]) cout << "error";
					}

				}
				setBeta(k,j,newBeta);
				setDelta(k,j, max(2.0 * abs(getBeta(k,j)), getDelta(k,j) / 2.0));

			}

					
	}
}

void Logistic::output(string fname)
{
	ofstream fres;
    fres.open(fname.c_str());
    fres << C << " " << M << endl;
    for (int c = 0; c < C; c++)
	{
		for (int j = 0; j < M; j++)
			fres << getBeta(c,j) << " ";
  		fres << endl;	
  	}
  	fres.close();
}

void Logistic::input(string fname)
{
	ifstream fin;
    fin.open(fname.c_str());
    
    string text = "";
    getline(fin, text); 
    istringstream oss(text);

    oss >> C;  
    
    oss >> M; 
    //cout << C << " " << M << endl; 
    Beta = new float [C * M];
    int i = 0;
    while (!fin.eof())
    {
        getline(fin, text); 
        istringstream oss(text);
    	float val;  
    	int j = 0; 
    	while(oss >> val)
    	{
    		setBeta(i,j,val);
     		j++;
     	}
     	i++;	
     }
 
  	fin.close();
}

int Logistic::pred(int i)
{
	
	float * p = new float [C];
    
	for (int c = 0; c < C; c++) p[c] = 0;
	
	for (int c = 0; c < C; c++)
	{
		for (int j = 0; j < M; j++)

		{ 
			
			p[c] += getBeta(c,j) * Training.getX(i,j);
		}
		p[c] = exp(p[c]);
				
	}
		
	int maxId = 0;
	float maxVal = 0;
	for (int c = 0; c < C; c++)
	{
		if (p[c] > maxVal)
		{
			maxVal = p[c];
			maxId = c;	
		}
	}
		
	return maxId + 1;
}

void Logistic::Classify(string fname)
{
   
	ofstream fres;

    fres.open(fname.c_str());
	for (int i = 0; i < N; i++)		
		fres << pred(i) << endl;
	fres.close();

}



float Logistic::Test(string xFileName, string yFileName, vector <int> idsTest)
{
	
	Testing = Data(xFileName, yFileName, idsTest); 
	int N_test = Testing.N;
	
	int correct = 0;
	int wrong = 0;
	for (int i = 0; i < N_test; i++)
	{		
		int cl = pred(i);
		
		if (cl == Testing.y[i])
			correct ++;
		else wrong ++;	
	} 
	
    float er = float(wrong) / float(correct + wrong);
    return er;
}




