#ifndef LOGISTIC_H
#define LOGISTIC_H

#include <string.h>
#include <set>
#include <stdlib.h>
#include <stdio.h>
#include <cmath>
#include <iostream>
#include <vector>
#include <map>
#include "newick.h"
#include "Tree.h"

using namespace std;

class Data{
    private: 
		
	    int totalN;
    public:  
        int * y;
        float * X;      
        int * Y;
    	int N;
		int M; 
		int C;
    	Data();
        Data(string xFileName, string yFileName, vector <int> idsTrain);
        Data(string xFileName, vector <int> idsTrain);
        void GetData(string xFileName, vector <int> idsTrain);
        void getClasses(string yFileName, vector <int> idsTrain);
        float getX(int i, int j) { return X[i * M + j];}
        void setX(int i, int j, float val) {X[i * M + j] = val;}
        int getY(int i, int j) {return Y[i * C + j];}
        void normalize();
        //~Data();

};


class Logistic{
    private: 
		Data Training;
		//spidir::Tree * tree;
		spidir::Node *root;
        spidir::Tree *tree;
		float Lambd;
		float *Beta;
		float *Delta;
		int N;
		int M; 
		int C;
		vector <int> * NonZeroData;
		vector <int> J;
		float MegaSum;
		float Penalty;
		int NInterNodes;
		vector <vector <int> > kids;
		vector <vector <int> > leaves;
		vector <int> Level;
		vector <float> Dist;
		int * llevel;	
		float * ddist;
		float * height;
		float * W;
		float * W_leaf;

		float * s;
		float * g;
		float * MultS;
		float * ScalarMult; 
		float * NumBetas;
		float * A;
		float * r1_plus;
		float * r1_minus;
		float * r2;
        float * E;
        float * XY;

        Data Testing;

    public:  
        Logistic(string xFileName, string yFileName, string tFileName, vector <int> idsTrain);
        Logistic(string xFileName,vector <int> idsTrain);
        void del();
        void Train(float _lambd);
        float getBeta(int i, int j){return Beta[i * M + j];}
        void setBeta(int i, int j, float val){Beta[i * M + j] = val;}

        float getDelta(int i, int j){return Delta[i * M + j];}
        void setDelta(int i, int j, float val){Delta[i * M + j] = val;}

        float getScalarMult(int i, int j){return ScalarMult[i * N + j];}
        void setScalarMult(int i, int j, float val){ScalarMult[i * N + j] = val;}

        float getNumBetas(int i, int j){return NumBetas[i * C + j];}
        void setNumBetas(int i, int j, float val){NumBetas[i * C + j] = val;}

        float getA(int i, int j){return A[i * NInterNodes + j];}
        void setA(int i, int j, float val){A[i * NInterNodes + j] = val;}

        float getr1_plus(int i, int j){return r1_plus[i * M + j];}
        void setr1_plus(int i, int j, float val){r1_plus[i *M + j] = val;}

        float getr1_minus(int i, int j){return r1_minus[i * M + j];}
        void setr1_minus(int i, int j, float val){r1_minus[i *M + j] = val;}

        float getr2(int i, int j){return r2[i * M + j];}
        void setr2(int i, int j, float val){r2[i *M + j] = val;}

        float getXY(int i, int j){return XY[i * C + j];}
        void addXY(int i, int j, float val){XY[i *C + j] += val;}


        void Learn(float wScheme);
        void LoadTheTree(float wScheme);
        void TreePass(spidir::Node *node, int level, float dist);
        float TreePassHeight(int node);
        void Weights(int node, int level);
        void Scalar();
        void Derivatives();
        void PartialSquareSums();
        void PartialSquareSumsRecursion(int v);
        void TreePassNumBetas(int node);
        void TreePassSum(int node, int c, float X1, float X2, float Y1, float Y2, float Z, float ZZ);

        float compute(int k, int j);
        void dl1(int k, int j, float &a, float &c);
        float eval(int K, int J, float step);

        float Test(string xFileName, string yFileName, vector <int> idsTest);
        int pred(int i);

        void output(string fname);
        void input(string fname);
        void Classify(string fname);
};
int GetM(string xFileName);
int GetC(string yFileName);



#endif
