#include "Testing.h"
#include "Logistic.h"
#include <iostream>
#include <cmath>
#include <fstream>
#include <string>
#include <sstream>

#include <vector>
#include <sys/stat.h>
#include <sys/types.h>



Testing::Testing(string xName, string yName, string tName, int _NTests, int _KFold, 
            string setsPrefix, string logsPrefix, string outprefix)
{
     
    
    xFileName = xName;
    yFileName = yName;
    tFileName = tName;
  
    NTests = _NTests;
    KFold = _KFold;  

    setFileName = new string [11];
    for (int i = 0; i < NTests; i++)
    {
        stringstream ss;
        ss << i;
        string stri;
        ss >> stri;
        setFileName[i] = setsPrefix + stri + ".txt";
    }
    cout << NTests << endl;
    detailedLogs = logsPrefix + "log_detailed.txt";
    avrLogs = logsPrefix + "log_avr.txt";
    resultLogs = logsPrefix + "log_result.txt";
    cout << detailedLogs << endl;

    nW = 1;
    W = new float [nW];
    W[0] = 0.4;
    nL = 1;
    L = new float [nL];
    L[0] = 1;
    

    ofstream fres;
    fres.open(resultLogs.c_str(), std::ios_base::app);
    float bestW = W[0];
    float bestVal = 1;
    for (int i = 0; i < nW ; i ++) 
    {
        float val = Test(xFileName,yFileName,tFileName, W[i]);
        if (val < bestVal)
        {
            bestVal = val;
            bestW = W[i];
        }

    }   
    
    fres << outprefix << "\t" << bestW << '\t' << bestVal << endl;
    fres.close();     
}

float Testing::Test (string xFileName, string yFileName, string FileName, float wScheme)
{
    //wScheme = -2.0;
    float bestLamb = L[0];
    float bestVal = 1;
   
    ofstream flog, alog;
    flog.open(detailedLogs.c_str(),std::ios_base::app);
    alog.open(avrLogs.c_str(),std::ios_base::app); 
    //fres.open(resultLogs, ios::out | ios::trunc );

    flog << "xFileName:\t" << xFileName << endl;
    flog << "yFileName:\t" << yFileName << endl;
    flog << "tFileName:\t" << tFileName << endl;
    
    flog << "NTests:\t" << NTests << endl;
    flog << "KFold:\t" << KFold << endl;
    //flog << "setsPrefix: " << setsPrefix << endl;
    flog << "W\tLambda\ti\tj\terror" << endl;
  
    for (int Lambd = 0; Lambd < nL; Lambd ++)
    {
        float er = 0;
        int n = 0;
        cout << NTests << endl;
        for (int i = 0; i < NTests; i++)
        {
            cout << "i " << i << endl;        
            for (int j = 0; j < KFold; j++)
            {
                vector <int> idsTest;
                vector <int> idsTrain;
                setSets(j,setFileName[i], idsTest, idsTrain);
                Logistic A = Logistic(xFileName, yFileName, tFileName, idsTrain);
                A.Train(L[Lambd]);
                A.Learn(wScheme);

                float e = A.Test(xFileName, yFileName,idsTest);
                A.del();
                er += e;                
                n += 1;

                flog << wScheme << "\t" << L[Lambd] << "\t" <<
                    i << "\t" << j << "\t" << e << endl;

            }            
        }
        flog << "Avr:\t" << wScheme << "\t" << L[Lambd] << "\t" 
                     << float(er) / n << endl;
        alog << wScheme << "\t" << L[Lambd] << "\t" 
                     << float(er) / n << endl;

        if (float(er) / n <= bestVal)
        {
            bestVal = float(er) / n;
            bestLamb = L[Lambd];
        }  
        //cout << L[Lambd] << " " << wScheme <<  " " << float(er) / n << endl;
 
      
    }
    flog << "Best Lambda:\t" << wScheme << "\t" << bestLamb << "\t" << bestVal << endl;
    flog.close();
    alog.close();
    return bestVal;
    //cout << x << " " << y << " " << t << " " << bestVal << " " << bestLamb   << endl;
}

void setSets(int j, string setFileName, vector <int> &idsTest, vector <int> &idsTrain) 
{
    
    ifstream f;
    f.open(setFileName.c_str());
    string text = "";
    
    while (!f.eof())
    {
        getline(f, text);   
           
        if (text[0] == '>')
        {
            istringstream oss(text);
            string word;
            oss >> word; oss >> word;

            if (atoi(word.c_str()) == j)
            {
                getline(f, text);   
                istringstream id1(text);
                while(id1 >> word)
                    idsTest.push_back(atoi(word.c_str())) ;
                getline(f, text);   
                istringstream id2(text);
                while(id2 >> word)
                    idsTrain.push_back(atoi(word.c_str())) ;
            }   
        }
    }

    f.close();
    
}
