#include <sstream>
#include <iostream>
#include <cstdlib>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>
#include "Testing.h"
#include "newick.h"
#include "Tree.h"
#include <stdio.h>      
#include <stdlib.h>     
#include <time.h>       

using namespace std;

void printHelp()
{
    cout << "Usage:" << endl;
    cout << "-h  --help\tPrint help message." << endl << endl;
    cout << "MetaPhyl can be run in two modes: training and testing (or classification of\nnew samples)." << endl << endl;
    cout << "-train\t\tTraining mode." << endl;
    cout << "-test\t\tTraining mode." << endl;
    cout << "-d\t\tClass labels file (training mode)." << endl;
    cout << "-t\t\tPhylogenetic tree in the Newick format file (training mode)." << endl;
    cout << "-o\t\tOutput file (training and testing modes)." << endl;
    cout << "-c\t\tInput file for the testing mode produced during training phase." << endl;
    cout << "-w\t\tWeight parameter (training mode), value from 0 to 1." << endl;
    cout << "-lambda\t\tregularization parameter (training mode)." << endl << endl;
    cout <<"Examples." << endl;
    cout <<"Training:" << endl;
    cout << "./MetaPhyl -train -d example/samples.txt -t example/tree.tre -l example/labels.txt ";
    cout << "-w 0.5 -lambda 1 -o example/out.txt" << endl << endl;
    cout <<"Testing:" << endl;
    cout << "./MetaPhyl -test -d example/samples.txt -c example/out.txt -o example/result.txt" << endl << endl;

}

int main(int argc, char **argv) 
{

    if (argc < 2) 
    { 
        printHelp();
    } 
    else 
    { 
        
        string mode = "", xName = "",  yName = "", tName = "", out = "", fin = "", fout = "";
        float lambd = 1, w = 0.5;
        for (int i = 1; i < argc; i++) 
        { 
           // if (i + 1 != argc) 
           
            string arg = argv[i];
           
            {
                if (arg == "-train") 
                {
                    if (mode == "test")
                    {
                        cout << "Please, use either training or testing mode." << endl;
                        exit(0);
                    }
                    else mode = "train";
                } 
                else if (arg == "--help")// || argv[i] == "-h" ) 
                {
                    cout << 123 << endl;
                    mode = "help";
                } 
                else if (arg == "-test") 
                {
                    if (mode == "train")
                    {
                        cout << "Please, use either training or testing mode." << endl;
                        exit(0);
                    }
                    else mode = "test";
                } 
                else if (arg == "-d") 
                {
                    xName = argv[i + 1];
                    i++;
                } 
                else if (arg == "-l") 
                {
                    yName = argv[i + 1];
                    i++;
                }
                else if (arg == "-t") 
                {
                    tName = argv[i + 1];
                    i++;
                }
                else if (arg == "-o") 
                {
                    fout = argv[i + 1];
                    i++;
                }
                else if (arg == "-c") 
                {
                    fin = argv[i + 1];
                    i++;
                }
                else if (arg == "-w") 
                {
                    w = atof(argv[i + 1]);
                    i++;
                }
                else if (arg == "-lambda") 
                {
                    lambd = atof(argv[i + 1]);
                    i++;
                }
                else 
                {
                    cout << "Invalid arguments, please try again" << endl;;
                    exit(0);
                }
            
            }

        }

        if (mode == "help")
        {
           printHelp();
        }
        else if (mode == "train")
        {
            vector <int> empty;
            Logistic A = Logistic(xName, yName, tName, empty);
            A.Train(lambd);
            A.Learn(w);
            A.output(fout);

        }
        else if (mode == "test")
        {
            vector <int> empty;
            Logistic A = Logistic(xName,empty);
            A.input(fin);
            A.Classify(fout);
        }
    }
       
}


