/*=============================================================================

  Matt Rasmussen
  Copyright 2007-2011

  General parsing functions

=============================================================================*/

// spidir headers
#include "parsing.h"

namespace spidir {


bool inChars(char c, const char *chars)
{
   if (!chars)
      return false;
   for (;*chars; chars++)
      if (c == *chars) return true;
   return false;
}


bool chomp(char *str)
{
   int len = strlen(str);
   if (len > 0 && str[len-1] == '\n') {
      str[len-1] = '\0';
      if (len > 1 && str[len-2] == '\r')
          str[len-2] = '\0';
      return true;
   } else
      return false;
}


vector<string> split(const char *str, const char *delim, bool multiDelim)
{
    vector<string> tokens;   
    int i=0, j=0;
   
    while (str[i]) {
        // walk to end of next token
        for (; str[j] && !inChars(str[j], delim); j++);
        
        if (i == j)
            break;

        // save token
        tokens.push_back(string(&str[i], j-i));
        
        if (!str[j])
            break;
        j++;
        i = j;
    }
    
    return tokens;
}


string trim(const char *word)
{
    char buf[101];
    if (sscanf(word, "%100s", buf) == 1)
        return string(buf);
    else
        return string("");
}


} // namespace spidir
